using System;
using System.Collections;
using System.IO;
using System.Web.Caching;
using System.Xml.Serialization;
using System.Xml.XPath;
using DotNetNuke.Common;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Services.Log.EventLog;
using DotNetNuke.Services.Scheduling;

namespace ITCrossing
{
	public class MetaPostInstaller : SchedulerClient
	{
		public MetaPostInstaller(ScheduleHistoryItem objScheduleHistoryItem)
			: base()
		{
			this.ScheduleHistoryItem = objScheduleHistoryItem;
		}

		public override void DoWork()
		{
			try
			{
				// Update the Friendly URLs configuration file.
				
				// First retrieve the rewriter rules
				RewriterRuleCollection rewriteRules = RewriterConfiguration.GetConfig().Rules;

				// Check to see if the rule already exists.  
				bool bExists = false;
				int ruleOldIndex = -1;
				int ruleCount = 0;
					
				foreach(RewriterRule ruleTemp in rewriteRules)
				{
					// The first version of metaPost used a different rule.  We'll remove this
					// old rule if it exists.
					if (ruleTemp.LookFor.ToLower() == ".*?//[^/]+/([^/]*)/metapost.ashx") ruleOldIndex = ruleCount;
					if (ruleTemp.LookFor.ToLower() == "(.*)/metapost.ashx") bExists = true;
					ruleCount ++;
				}

				if (ruleOldIndex != -1)
				{
					// Remove the old rule as noted above.
					rewriteRules.RemoveAt(ruleOldIndex);
				}

				// If we don't already have our entry, add it.
				if (!bExists)
				{
					// Create a new RewriterRule
					RewriterRule newRule = new RewriterRule();

					// Set the properties for the new rule
					newRule.LookFor = "(.*)/metapost.ashx";
					newRule.SendTo = "~/metapost.ashx?calias=$1";

					// Add the new rule to the collection
					rewriteRules.Add(newRule);

					// Persist the RewriterConfiguration
					RewriterConfiguration.SaveConfig(rewriteRules);
				}

				// Remove Scheduler entries since we've 
				// accomplished our goal of adding a new rewrite rule.
				string sqlDelete = "DELETE FROM {databaseOwner}[{objectQualifier}ScheduleHistory] " +
				                   "	FROM {databaseOwner}[{objectQualifier}ScheduleHistory] sh " +
								   "	JOIN {databaseOwner}[{objectQualifier}Schedule] s " +
								   "		ON sh.ScheduleID = s.ScheduleID " +
				                   "	WHERE s.TypeFullName = 'ITCrossing.MetaPostInstaller,itcMetaPost' " +
								   "DELETE FROM {databaseOwner}{objectQualifier}Schedule WHERE TypeFullName = 'ITCrossing.MetaPostInstaller,itcMetaPost'";

				DotNetNuke.Data.DataProvider.Instance().ExecuteSQL(sqlDelete);
				
				// We would normally write success here, but if we have succeeded, we should have
				// removed our entries in the Scheduler tables, so trying to write a ScheduleHistory line 
				// would fail.  Left for reference.

				//this.ScheduleHistoryItem.Succeeded = true;
				//this.ScheduleHistoryItem.AddLogNote("Successfully installed the metaPost Friendly URLs" +
				//  DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString());
			}
			catch (Exception exc)
			{
				this.ScheduleHistoryItem.Succeeded = false;
				this.ScheduleHistoryItem.AddLogNote("metaPost Friendly URLs installation failed.  Please install a frienldy URL which looks for .*/(.*)/metapost.ashx and sends to ~/metapost.ashx?calias=$1");
				this.Errored(ref exc);
			}
		}

		// Since the location of the rewriter configuration classes 
		// has changed between 4.5 and 4.8 and since the code is identical
		// the functionality is repeated here.

		# region Rewriter Code 

		//  Original DNN Copyright notice since code is being reused here

		//' DotNetNuke - http://www.dotnetnuke.com
		//' Copyright (c) 2002-2007
		//' by DotNetNuke Corporation
		//'
		//' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
		//' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
		//' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
		//' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
		//'
		//' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
		//' of the Software.
		//'
		//' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
		//' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
		//' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
		//' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
		//' DEALINGS IN THE SOFTWARE.
		[Serializable, XmlRoot("RewriterConfig")]
		public class RewriterConfiguration
		{
			private RewriterRuleCollection _rules;
 
			public static RewriterConfiguration GetConfig()
			{
				RewriterConfiguration Config = new RewriterConfiguration();
				Config.Rules = new RewriterRuleCollection();
				FileStream fileReader = null;
				string filePath = "";
				try
				{
					// Our implementation will have to ignore this attempt to access the cache
					//Config = (RewriterConfiguration)DataCache.GetCache("RewriterConfig");
					//if (Config != null)
					//{
					//    return Config;
					//}
					filePath = Globals.ApplicationMapPath + @"\SiteUrls.config";
					if (!File.Exists(filePath) && File.Exists(Globals.ApplicationMapPath + @"\Config\SiteUrls.config"))
					{
						File.Copy(Globals.ApplicationMapPath + @"\Config\SiteUrls.config", Globals.ApplicationMapPath + @"\SiteUrls.config", true);
					}
					fileReader = new FileStream(filePath, FileMode.Open, FileAccess.Read, FileShare.Read);
					XPathDocument doc = new XPathDocument(fileReader);
					Config = new RewriterConfiguration();
					Config.Rules = new RewriterRuleCollection();
					foreach (XPathNavigator nav in doc.CreateNavigator().Select("RewriterConfig/Rules/RewriterRule"))
					{
						RewriterRule rule = new RewriterRule();
						rule.LookFor = nav.SelectSingleNode("LookFor").Value;
						rule.SendTo = nav.SelectSingleNode("SendTo").Value;
						Config.Rules.Add(rule);
					}

					// Ignore caching code since this will only be called once.
					//if (File.Exists(filePath))
					//{
					//    CacheDependency dep = new CacheDependency(filePath);
					//    DataCache.SetCache("RewriterConfig", Config, dep);
					//}
				}
				catch (Exception ex)
				{
					EventLogController objEventLog = new EventLogController();
					LogInfo objEventLogInfo = new LogInfo();
					objEventLogInfo.AddProperty("UrlRewriter.RewriterConfiguration", "GetConfig Failed");
					objEventLogInfo.AddProperty("FilePath", filePath);
					objEventLogInfo.AddProperty("ExceptionMessage", ex.Message);
					objEventLogInfo.LogTypeKey = EventLogController.EventLogType.HOST_ALERT.ToString();
					objEventLog.AddLog(objEventLogInfo);
				}
				finally
				{
					if (fileReader != null)
					{
						fileReader.Close();
					}
				}
				return Config;
			}

			public static void SaveConfig(RewriterRuleCollection rules)
			{
				if (rules != null)
				{
					RewriterConfiguration config = new RewriterConfiguration();
					config.Rules = rules;
					XmlSerializer ser = new XmlSerializer(typeof(RewriterConfiguration));
					string filePath = Globals.ApplicationMapPath + @"\SiteUrls.config";
					if (File.Exists(filePath))
					{
						File.SetAttributes(filePath, FileAttributes.Normal);
					}
					FileStream fileWriter = new FileStream(filePath, FileMode.Create, FileAccess.Write, FileShare.Write);
					StreamWriter writer = new StreamWriter(fileWriter);
					ser.Serialize((TextWriter)writer, config);
					writer.Close();
					fileWriter.Close();
					
					// Ignore caching code for this particular use of the DNN code - see notes above
					//CacheDependency dep = new CacheDependency(filePath);
					//DataCache.SetCache("RewriterConfig", config, dep);
				}
			}
			public RewriterRuleCollection Rules
			{
				get
				{
					return this._rules;
				}
				set
				{
					this._rules = value;
				}
			}
 

		}
 
		[Serializable()]
		public class RewriterRule
		{
			private string _lookFor;
			private string _sendTo;

			public string LookFor
			{
				get
				{
					return this._lookFor;
				}
				set
				{
					this._lookFor = value;
				}
			}
			public string SendTo
			{
				get
				{
					return this._sendTo;
				}
				set
				{
					this._sendTo = value;
				}
			}
		}

		[Serializable()]
		public class RewriterRuleCollection : CollectionBase
		{
			public void Add(RewriterRule r)
			{
				this.InnerList.Add(r);
			}

			public virtual RewriterRule this[int index]
			{
				get
				{
					return (RewriterRule)base.List[index];
				}
				set
				{
					base.List[index] = value;
				}
			}


		}

		# endregion

	}
}
